@@ -7,7 +7,7 @@ use Perl::Critic::Utils qw(:severities :classification :ppi);
 use Path::Tiny 'path';
 use parent 'Perl::Critic::Policy';
 
-our $VERSION = '0.023';
+our $VERSION = '0.024';
 
 use constant DESC => 'No package matching the module filename';
 use constant EXPL => 'A Perl module file is expected to contain a matching package name, so it can be used after loading it from the filesystem. A module file that doesn\'t contain a matching package name usually indicates an error.';
@@ -23,20 +23,16 @@ sub violates {
 	my $packages = $elem->find('PPI::Statement::Package') || [];
 	
 	my $filepath = path($doc->filename)->realpath;
-	my @path_parts;
-	unshift @path_parts, $filepath->basename(qr/\.pm/);
-	while (!$filepath->is_rootdir) {
-		$filepath = $filepath->parent;
-		unshift @path_parts, $filepath->basename;
-	}
+	my $basename = $filepath->basename(qr/\.pm/);
+	$filepath = $filepath->parent->child($basename);
 	
 	my $found_match;
 	PKG: foreach my $package (@$packages) {
 		my $namespace = $package->namespace;
-		my @path_copy = @path_parts;
+		my $path_copy = $filepath;
 		foreach my $part (reverse split '::', $namespace) {
-			next PKG unless $part eq $path_copy[-1];
-			pop @path_copy;
+			next PKG unless $part eq $path_copy->basename;
+			$path_copy = $path_copy->parent;
 		}
 		$found_match = 1;
 		last;
